#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""GnuPG SSSS.
    Generate a GnuPG private and public key, then encrypt the private key with
    AES256, and split the password with the Shamir's Secret Sharing algorithme.
    The shares are the encrypted with the public keys of 'n' recipients.
    
    This methodology allows you to have a private/public key, without knowing
    the private key ab-initio. To unlock the private key, at least 't' shares
    have to be used, hence at least 't' of the recipients have to be aware to be
    able to decrypt the data.
    
    The data can always be encrypted with the public key since always available
    to anyone.
    
                                                                Marco De Donno

Usage:
    gpgssss.py generate (<name> | --name <name>) (<mail> | --mail <mail>) [(<comment> | --comment <comment>)] [options]
    gpgssss.py combine <shares>...
    gpgssss.py combine -f <sharesfolder>
    gpgssss.py -h | --help
    gpgssss.py --version

Options:
    -n --name name          Name of the private key
    -m --mail mail          Mail address of the private key
    -c --comment comment    Comment of the private key
    
    -l --key_length length  Key length [default: 4096]
    -c --key_type type      Key type [default: RSA]
    
    -t --threshold t        Threshold to reach to be able to unlock the private key. If not
                            set, will be set to 'n' (all keys are mendatory to unlock the
                            private key). [default: All]
    
    -o --output output      Output directory to store the produced files [default: /output]
    -p --public public      Directory containing the public keys to load [default: /public]
    
    -h --help               Show this screen
    --version               Show version

"""

from docopt import docopt
from functions import *

if __name__ == '__main__':
    args = docopt( __doc__, version = 'GPGSSSS' )
    
    if args[ 'generate' ]:
        name = args.get( "<name>" ) or args.get( "--name" )
        mail = args.get( "<mail>" ) or args.get( "--mail" )
        comment = args.get( "<comment>" ) or args.get( "--comment" ) or name
        
        k = gen_keys( name, comment, mail )
        password = gen_password()
        private = encrypt_private( k[ 'private' ], password )
        private = private.replace( "\r", "" )
        public = k[ 'public' ]
        public = public.replace( "\r", "" )
        
        t = args.get( "--threshold" )
        if t == "All":
            t = len( os.listdir( args.get( "--public" ) ) )
        t = int( t )
        
        with open( args.get( "--output" ) + "/private.asc", "w+" ) as fp:
            fp.write( private )
        
        with open( args.get( "--output" ) + "/public.asc", "w+" ) as fp:
            fp.write( public )
        
        split_and_encrypt( password, t, args.get( "--public" ), args.get( "--output" ) )
        
    elif args[ 'combine' ]:
        if args[ '<shares>' ] != []:
            print unsplit_password( args[ '<shares>' ] )
        
        elif args[ '<sharesfolder>' ] != None:
            shares = []
            for f in os.listdir( args[ '<sharesfolder>' ] ):
                with open( args[ '<sharesfolder>' ] + "/" + f, "r" ) as fp:
                    s = fp.readline()
                    if s.count( "-" ) == 1:
                        shares.append( s )
            
            print unsplit_password( shares )
