#!/usr/bin/python
# -*- coding: UTF-8 -*-

import base64
import hashlib
import os
import string
import time
import uuid

import gnupg
import netifaces
import psutil
import secretsharing


def ip4_addresses():
    """
        Return a list with alll IPv4 addresses of the host.
    """
    ip_list = []
    try:
        for interface in netifaces.interfaces():
            for link in netifaces.ifaddresses( interface )[ netifaces.AF_INET ]:
                ip_list.append( str( link[ 'addr' ] ) )
    except:
        pass
    
    return ip_list

def gen_password():
    """
        Generate a strong password, based upon unique information regarding the
        host generating the password (uuid1 and list of IPs), generation time
        (UNIX time in miliseconds and uptime in miliseconds), and some random
        data from the /dev/urandom device (1024 bits)). This input data is then
        passed to the sha512 function to uniformize the output (fix length and
        charset).
    """
    lst = []
    lst.append( uuid.uuid1() )
    lst.append( "-".join( ip4_addresses() ) )
    lst.append( time.time() )
    lst.append( psutil.boot_time() )
    lst.append( base64.b64encode( os.urandom( 1024 ) ) )
    lst = "/".join( map( str, lst ) )
    
    return hashlib.sha512( lst ).hexdigest()

def split_password( password, t, n ):
    """
        Split the password passed in parameter with the Shamir's Secret Sharing
        algorithm. The threshold (t) and total number of shares (n) parameters
        can be set as function argument.
        
        Since the password can not start with a '0' string, the 'p/' string is
        prepended to the actual password. This string will be removed in the
        unsplit_password().
    """
    password = "p/" + password
    
    sharer_class = secretsharing.SecretSharer
    sharer_class.share_charset = string.hexdigits + 'p/'
    sharer_class.secret_charset = string.hexdigits + 'p/'
    
    return sharer_class.split_secret( password, t, n )

def unsplit_password( shares ):
    """
        Function to combine the available shares to generate the password.
        
        Since the password can not start with a '0' string, the 'p/' string is
        prepended to the actual password. This string has to be removed in this
        function.
    """
    sharer_class = secretsharing.SecretSharer
    sharer_class.share_charset = string.hexdigits + 'p/'
    sharer_class.secret_charset = string.hexdigits + 'p/'
    
    password = sharer_class.recover_secret( shares )
    password = password.replace( "p/", "" )
    
    return password

def gen_keys( name, comment, mail, key_type = "RSA", length = 4096 ):
    """
        Generate and return UNPROTECTED private/public keys.
        The .gnupg folder is still protected by a random password, not stored anywhere.
    """
    gpg = gnupg.GPG()
    
    dummy_password = gen_password()
    
    input_data = gpg.gen_key_input( 
        name_real = name,
        name_comment = comment,
        name_email = mail,
        key_type = key_type,
        passphrase = dummy_password,
        key_length = length
     )
    
    key = gpg.gen_key( input_data )
    
    ascii_armored_public_keys = gpg.export_keys( str( key ) )
    ascii_armored_private_keys = gpg.export_keys( str( key ), True, passphrase = dummy_password )
    
    return {
        "public": ascii_armored_public_keys,
        "private": ascii_armored_private_keys
    }

def encrypt_private( data, password, algo = 'AES256' ):
    """
        Encrypt the data passed in argument with the password. By default, the
        Cipher used is AES256.
    """
    return str( gnupg.GPG().encrypt( data, recipients = None, symmetric = algo, passphrase = password, armor = True ) )


def encrypt_to_destinator( shares, keys_folder, outfolder ):
    """
        Encrypt the shares passed in argument with the public keys located in
        the 'keys_folder' location.
        
        TODO: Check if the number of public keys is compatible with the number
        of shares, and act accordingly.
    """
    gpg = gnupg.GPG()
    
    for k in gpg.list_keys():
        gpg.delete_keys( fingerprints = k[ 'fingerprint' ] )
    
    recipients = []
    for f in os.listdir( keys_folder ):
        name, ext = os.path.splitext( f )
        recipients.append( name )
        
        with open( keys_folder + "/" + f ) as fp:
            gpg.import_keys( fp.read() )
    
    if len( shares ) != len( recipients ):
        raise Exception( "The number of public keys has to be the same as the number of shares !" )
    
    for k in gpg.list_keys():
        gpg.trust_keys( k[ 'fingerprint' ], "TRUST_ULTIMATE" )
        
    if outfolder == "stdout":
        ret = {}
        for s, k in zip( shares, recipients ):
            ret[ k ] = str( gpg.encrypt( s, k ) ).replace( "\r", "" )
            
        return ret
    
    else:
        for s, k in zip( shares, recipients ):
            with open( outfolder + "/" + str( k ) + ".txt", "w+" ) as fp:
                data = str( gpg.encrypt( s, k ) )
                data = data.replace( "\r", "" )
                fp.write( data )
        
        return os.listdir( outfolder )
    
def split_and_encrypt( password, t, keys_folder, outfolder ):
    s = split_password( password, t, len( os.listdir( keys_folder ) ) )
    return encrypt_to_destinator( s, keys_folder, outfolder )
